import requests
import sys


class Room:

    def get_all_rooms(self):
        """
        Gives a list of all the rooms the specific bot is in
        this request uses url https://api.ciscospark.com/rooms 
        details on the rooms URL can be found in: https://developer.webex.com/docs/api/v1/rooms/list-rooms
        """
        url_route = "rooms"
        data = requests.get(self.URL + url_route, headers=self.headers)
        return data

    def create_room(self, title=None, team_id=None, room_type=None):
        """
        Creates a room within a team, also known as a space. This will be within the team with ID teamId
        this request uses url https://api.ciscospark.com - POST request
        details on the create rooms can be found in https://developer.webex.com/docs/api/v1/rooms/create-a-room 
        """

        if title is None:
            sys.exit("'title' is a required field")
    
        if team_id is None:
            sys.exit("'teamId; is a required field")

        url_route = "rooms"

        json = {
            "teamId": team_id,
            "title": title
        }

        if room_type is not None:
            json["type"] = room_type

        data = requests.post(self.URL + url_route, json=json, headers=self.headers)
        return data

    def get_room_details(self, room_id=None):
        """
        GETS DETAILS OF A PARTICULAR ROOM
        request uses url https://api.ciscospark.com/{roomId} - GET request
        details on the get room details can be found in https://developer.webex.com/docs/api/v1/rooms/get-room-details 
        """

        if room_id is None:
            sys.exit("'roomId' is a required field")

        url_route = "rooms"

        data = requests.get(self.URL + url_route + "/" + room_id, headers=self.headers)
        return data

    def update_room_details(self, room_id=None, title=None):
        """
        UDPATES THE DETAILS OF A PARTICULAR ROOM based on the **kwargs given by the user
        request uses url https://api.ciscospark.com/{roomId} - PUT request
        details on the update room details can be found in https://developer.webex.com/docs/api/v1/rooms/update-a-room
        """

        if room_id is None:
            sys.exit("'roomId' is a required field")

        elif title is None:
            sys.exit("'title' is a required field")

        json = {
            "title": title
        }

        url_route = "rooms"

        data = requests.put(self.URL + url_route + '/' + room_id, json=json, headers=self.headers)
        return data

    def delete_room(self, room_id=None):
        """
        DELETES A ROOM with ID roomId
        uses url https://api.ciscospark.com/v1/rooms/{roomId}
        details on the delete room can be found in https://developer.webex.com/docs/api/v1/rooms/delete-a-room
        """

        if room_id is None:
            sys.exit("'roomId' is a required field")

        url_route = "rooms"
        data = requests.delete(self.URL + url_route + "/" + room_id, headers=self.headers)
        return data







import sys
import requests


class People:

    def get_people(self, email=None):
        """
        gets a list of people with a particular attribute
        uses https://api.ciscospark.com/people - GET request
        """

        if email is None:
            sys.exit("'email' is a required field")

        url_route = "people"

        params = {
            "email": email
        }
        data = requests.get(self.URL + url_route, headers=self.headers, params=params)
        return data

    def get_person_details(self, person_id=None):
        """
        returns specific information of the person with ID personId
        uses https://api.ciscospark.com/people/{ personId } - GET request
        API reference can be found in: https://developer.webex.com/docs/api/v1/people/get-person-details 
        """

        if person_id is None:
            sys.exit("'personId' is a required field")

        url_route = "people"

        data = requests.get(self.URL + url_route + "/" + person_id, headers=self.headers)
        return data

    def get_own_details(self):
        """
        gets the bots own information
        uses https://api.ciscospark.com/people - GET request
        API reference can be found in: https://developer.webex.com/docs/api/v1/people/get-my-own-details 
        """

        url_route = "people/me"

        data = requests.get(self.URL + url_route, headers=self.headers)
        return data



from python_webex.v1.Card import Card
import requests 
import sys

class Message:
    """
    Message requests uses URL https://api.ciscospark.com/v1/messages

    Enables us to interact with the Messages in the Cisco Webex platform. 
    sending messages, listing your messages etc.  
    """
    
    def send_message(self, room_id=None, text=None, files=[]):
        """
        Allows for one to send a message to a room
        details on the rooms URL parameters can be found in https://developer.webex.com/docs/api/v1/messages/create-a-message
        'files' is a list of the files(images, audios etc) you want to send to the user, if the user wants to attach files with the message
        
        Arguments:
		    room_id => This is a 
		    text => The text being sent in the message 
            files => A list of files you want to sell. Each element in the list is a directory path to the file. e.g files=['/this/is/my/path/this_image.jpg', 'this/is/my/second/path/this_pdf.pdf']
        """
        if room_id == None:
            sys.exit("'roomId' is a required field")
        
        if text == None:
            sys.exit("'text' is a required field")

        if type(files) != list:
            sys.exit("'files' needs to be a list")

        url_route = "messages"

        data = {
            "roomId": room_id,
            "text": text,
        }

        if len(files) > 0:
            data["files"] = files
        
        data = requests.post( self.URL + url_route, headers=self.headers, json=data )
        return data
    
    def send_card(self, card:Card,  room_id:str, markdown: str="[This is the default markdown title]"):
        """
		Cars are elements that can hold forms and improve interactivity of the messages. 
        For example, if you are using a bot to monitor your networking devices, this will require you to login the networking devices first. 
        You can send a form for one to login to the networking devices. 
        """
        
        message = {
            "roomId": room_id,
            "markdown": markdown,
            "attachments": card.content
        }
        
        url_route = "messages"
        data = requests.post(self.URL + url_route, headers=self.headers, json=message)
        return data
    
    def get_attachment_response(self, attachment_id: str):
        """
        Gets the response for when an attachment has been sent
        """
        url_route = "attachment/actions/{}".format(attachment_id)

        response = requests.get(self.URL + url_route, headers=self.headers)
        return response.json()

    def get_messages(self, room_id=None):
        """
        gets all the messages sent and received in a specific room
        details on the list-messages URL parameters can be found in https://developer.webex.com/docs/api/v1/messages/list-messages
        """

        if room_id == None:
            sys.exit("'room_id' is a required field")
        
        url_route = "messages"

        params = {
            "roomId": room_id
        }
        data = requests.get( self.URL + url_route, headers=self.headers, params=params )
        return data
    
    def get_direct_messages(self, person_id=None):
        """
        gets a list of all the messages sent in 1 to 1 rooms. This is basically a list all the DMs :)
        details on the list-direct-messages URL parameters can be found in https://developer.webex.com/docs/api/v1/messages/list-direct-messages 
        """
        
        if person_id == None:
            sys.exit("'person_id' is a mandatory field")

        url_route = "messages"

        params = {
            "personId": person_id
        }
        data = requests.get( self.URL + url_route + "/direct", headers=self.headers, params=params )
        return data
    
    def get_message_details(self, message_id=None):
        """
        gets details of a specific message
        e.g roomId, roomType, created, mentionedPeople ...
        details on the get message details URL parameters can be found in https://developer.webex.com/docs/api/v1/messages/get-message-details
        """

        if message_id == None:
            sys.exit("'message_id' is a required field")
        
        url_route = "messages/" + message_id

        data = requests.get( self.URL + url_route, headers=self.headers)
        return data

    def delete_message(self, message_id=None):
        """
        deletes a message with ID messageId
        details on the delete message URL can be found in https://developer.webex.com/docs/api/v1/messages/delete-a-message
        """

        if message_id == None:
            sys.exit("'message_id' is not a required field")
        
        url_route = "messages/" + message_id

        data = requests.delete( self.URL + url_route, headers=self.headers )
        return data



from python_webex.v1.People import People
from python_webex.v1.Room import Room
from python_webex.v1.Webhook import Webhook
from python_webex.v1.Message import Message

import os
import sys


class Bot(People, Room, Webhook, Message):

    def __init__(self):

        # declare headers and how the token will be gotten from the system
        self.URL = "https://api.ciscospark.com/"
        self.auth_token = os.getenv("auth_token")

        if self.auth_token == None:
            sys.exit("'auth_token' not set in the environment variables")
        
        self.headers = {
            "Authorization": "Bearer " + self.auth_token,
            "Content-Type": "application/json"
        }
        
        # self.hears to function maps when a specific word is heard to a function
        # for example, when one says 'hi' and you want to map it to say_hi() function
        self.hears_to_function = {

        }

        self.hears_file_to_function = {
            
        }

        self.attachment_response_to_function = {

        }

        # default attachment variable will hold the function that is supposed to be the default action whenever an attachment is sent to the bot
        self.default_attachment = None

        # maps what will happen when a file is received with a particular type of text
        self.hears_file_to_function = {

        }

        self.attach_function = None

    """
    decorator meant to do a specific action when called
    """
    def on_hears(self, message_text):
        def hear_decorator(f):
            self.hears_to_function[message_text] = f
        return hear_decorator
    
    """
    decorator waiting for any attachment that has been sent to the bot
    """
    def set_default_file_response(self):
        def hear_default_file_attachment(f):
            self.default_attachment = f
        return hear_default_file_attachment
    
    """
    decorator for when a file is received with a particular type of text. 
    """
    def set_file_action(self, message_text: str):
        def hear_file_attachment(f):
            self.hears_file_to_function[message_text] = f
        return hear_file_attachment

    """
    decorator waiting for an attachment to be filled and returned for it to do a specific action
    """
    def attachment_response(self, message_id: str):
        def response_decorator(f):
            self.attachment_response_to_function[message_id] = f
        return response_decorator



import requests
import sys


class Webhook:

    def get_all_webhooks(self):
        """
        GETS A LIST OF ALL THE WEBHOOKS CURRENTLY CONNECTED TO YOUR BOT
        uses the https://api.ciscospark.com/v1/webhooks - GET request
        details on the list webhooks URL can be found in https://developer.webex.com/docs/api/v1/webhooks/list-webhooks
        """

        url_route = "webhooks"

        data = requests.get(self.URL + url_route, headers=self.headers)

        return data

    def create_webhook(self, name=None, target_url=None, resource=None, event=None):
        """
        Enables one to create a webhook that will be listening to events sent to the bot
        uses the https://api.ciscospark.com/v1/webhooks - POST request
        details on create webhooks URL can be found in https://developer.webex.com/docs/api/v1/webhooks/create-a-webhook
        """

        url_route = "webhooks"

        if name is None:
            sys.exit("'name' is a required field")

        elif target_url is None:
            sys.exit("'targetUrl' is a required field")

        elif resource is None:
            sys.exit("'resource' is a required field")

        elif event is None:
            sys.exit("'event' is a required field")

        # check for if a webhook with this URL already exists for this particular bot
        # cause apparently Cisco does not do that for us when creating webhooks. But tis all good :)
        existing_webhooks = self.get_all_webhooks().json()
        for webhook in existing_webhooks['items']:
            if webhook['targetUrl'] == target_url:
                return self.get_webhook_details(webhook_id=webhook['id'])

        json = {
            "name": name, "targetUrl": target_url, "resource": resource, "event": event
        }

        data = requests.post(self.URL + url_route, headers=self.headers, json=json)
        return data

    def delete_webhook(self, webhook_id=None):
        """
        Deletes a webhook that has ID webhookId
        uses the https://api.ciscospark.com/webhooks - DELETE request
        details on delete webhooks URL can be found in https://developer.webex.com/docs/api/v1/webhooks/delete-a-webhook 
        """

        url_route = "webhooks"

        if webhook_id is None:
            sys.exit("'webhookId' is a required field")

        data = requests.delete(self.URL + url_route + "/" + webhook_id, headers=self.headers)
        return data

    def update_webhook(self, webhook_id=None, name=None, target_url=None):
        """
        'name' is the updated name of the webhook
        'targetUrl' is the updated targetUrl of the webhook

        Edit a webhook with ID of webhookId
        uses the https://api.ciscospark.com/webhooks - PUT request
        details on edit webhook URL can be found in https://developer.webex.com/docs/api/v1/webhooks/update-a-webhook
        """

        url_route = "webhooks"

        if webhook_id is None:
            sys.exit("'webhookId' is a required field")

        elif name is None:
            sys.exit("'name' is a required field")

        elif target_url is None:
            sys.exit("'targetUrl' is a required field")

        json = {
            "name": name, "targetUrl": target_url
        }

        data = requests.put(self.URL + url_route + "/" + webhook_id, json=json, headers=self.headers)
        return data

    def get_webhook_details(self, webhook_id=None):
        """
        Get the details of a single webhook with id of webhookId
        uses https://api.ciscospark.com/webhooks/{roomId} - GET request
        details on get webhook details URL can be found in https://developer.webex.com/docs/api/v1/webhooks/get-webhook-details 
        """

        url_route = "webhooks"

        if webhook_id is None:
            sys.exit("'webhookId' is a required field")

        data = requests.get(self.URL + url_route, headers=self.headers)
        return data



from flask import Flask, request
from pprint import pprint
import inspect

app = Flask(__name__)
bot = None

@app.route("/", methods=[ 'GET', 'POST' ])
def index():
    json_data = request.get_json()
    message_id = json_data[ "data" ][ "id" ]
    message_info = bot.get_message_details( message_id=message_id ).json()
    if message_info[ "personId" ] == bot.get_own_details().json()[ 'id' ]:
        return "cannot respond to my own messages"
    

    if 'files' in message_info:
        if 'text' in message_info:
            if message_info['text'] in bot.hears_file_to_function:
                message_text = message_info['text']
                files = message_info['files']

                if 'message_info' in bot.hears_to_function[message_text].__code__.co_varnames:
                    bot.hears_file_to_function[message_text](files=message_info['files'], room_id=message_info['roomId'], message_info=message_info)
                else:
                    bot.hears_file_to_function[message_text](files=message_info['files'], room_id=message_info['roomId'])

                return "Works"
            elif '*' in bot.hears_file_to_function:

                if 'message_info' in bot.hears_to_function[message_text].__code__.co_varnames:
                    bot.hears_file_to_function['*'](files=message_info['files'], room_id=message_info['roomId'], message_info=message_info)
                else:
                    bot.hears_file_to_function['*'](files=message_info['files'], room_id=message_info['roomId'])

                return "Default file action"
            else:
                print("Default response for file sent with text not set")

        elif bot.default_attachment is not None:
            if 'message_info' in bot.hears_to_function[message_text].__code__.co_varnames:
                bot.default_attachment(files=message_info['files'], room_id=message_info['roomId'], message_info=message_info)
            else:
                bot.default_attachment(files=message_info['files'], room_id=message_info['roomId'])
            return "Works"
        else:
            print("No action set for receiving the file with text '{}'".format( message_info['text'] ))

    elif message_info[ "text" ].strip() != "" and message_info[ "text" ] in bot.hears_to_function:
        message_text = message_info[ "text" ]
        # bot.hears_to_function[ message_text ](room_id=message_info["roomId"])
        
        if 'message_info' in bot.hears_to_function[message_text].__code__.co_varnames:
            bot.hears_to_function[ message_text ](room_id=message_info["roomId"], message_info=message_info)
        else:
            bot.hears_to_function[ message_text ](room_id=message_info["roomId"])
        

    elif message_info["text"].strip() != "" and  message_info[ "text" ] not in bot.hears_to_function:
        
        if 'message_info' in bot.hears_to_function[message_text].__code__.co_varnames:
            bot.hears_to_function[ "*" ]( room_id=message_info["roomId"], message_info=message_info)
        else:
            bot.hears_to_function[ "*" ]( room_id=message_info["roomId"])
        
    return "successfully responded"



@app.route("/attachment-response", methods=["GET", "POST"])
def attachment_response():
    json_data = request.get_json()
    message_id = json_data['data']['messageId']
    message_dict = bot.get_attachment_response(json_data['data']['id'])
    if message_id in bot.attachment_response_to_function:
        response = bot.attachment_response_to_function[message_id](message_dict)
    else:
        room_id = message_dict['roomId']
        bot.send_message(room_id=room_id, text='The form could not be submitted. You may need to request for the form again then submit. Sorry for the inconvenience :)')

    return "attachment response received"



import setuptools

required = None
with open('requirements.txt') as f:
    required = f.read().splitlines()


with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="python_webex_bot",
    version="0.80",
    scripts=["python_webex_bot"],
    author="Paul-weqe",
    author_email="paul1tw1@gmail.com",
    description="Remove unused dependencies from requirements.txt",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Paul-weqe/python_webex_bot",
    packages=setuptools.find_packages(),
    install_requires=[
        "certifi",
        "chardet",
        "click",
        "Flask",
        "idna",
        "itsdangerous",
        "Jinja2",
        "MarkupSafe",
        "requests",
        "urllib3",
        "Werkzeug"
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent"
    ],
)



import sys
import requests


"""
Cards give a better level of interactivity to the Webex Platform. 
They can allow us to create form like interactive interfaces from the bot.
"""

class Card:
    def __init__(self):
        self.content = [
            {
                "contentType": "application/vnd.microsoft.card.adaptive",
                "content": {
                    "$schema": "http://adaptivecards.io/schemas/adaptive-card.json",
                    "type": "AdaptiveCard",
                    "version": "1.0",
                    "body": [
                        {
                            "type": "ColumnSet",
                            "columns": [
                                {
                                    "type": "Column",
                                    "width": 3,
                                    "items": []
                                }]
                        }
                    ],
                    "actions": [
                    ]
                }
            }
        ]

    def __add_element(self, element_dict):
        self.content[0]["content"]["body"][0]["columns"][0]["items"].append(element_dict)
    
    def __add_action(self, action_dict):
        self.content[0]["content"]["actions"].append(action_dict)
    
    def get_all_input_ids(self):
        inputs = []
        items = self.content[0]["content"]["body"][0]["columns"][0]["items"]
        for item in items:
            if "id" in item.keys():
                inputs.append(item["id"])
        return inputs
    
    def check_if_id_exists(self, id):
        ids = self.get_all_input_ids()
        items = self.content[0]["content"]["body"][0]["columns"][0]["items"]
        if id in ids:
            for item in items:
                if item['id'] == id:
                    items.remove(item)
    
    def check_if_submit_button_exists(self):
        actions = self.content[0]["content"]["actions"]
        for action in actions:
            if action["type"] == "Action.Submit":
                actions.remove(action)
    

    """
    Textblock is essentially normal text in html. as how you would put a <p>Here is the text</p>
    The TextBlock attributes and their explanations can be found at: https://adaptivecards.io/schemas/adaptive-card.json#/definitions/TextBlock
    """
    def add_text_block(
        self, text: str, is_subtle: bool = False, size: str = "default", weight: str = "default", wrap: bool = False
    ):
        element = {
            "type": "TextBlock",
            "text": text,
            "isSubtle": is_subtle,
            "size": size,
            "weight": weight,
            "wrap": wrap
        }
        self.__add_element(element)

    
    """
    add_input_text() adds the traditional text fields as they are used in normal forms. interprated as <input type="text"> when it comes to html
    The various attribute values can be found at: https://adaptivecards.io/schemas/adaptive-card.json#/definitions/Input.Text 
    Have a look at the URL above for more details on how the field is used
    """
    def add_input_text(
        self, input_id:str, input_is_multiline:bool = False, input_max_length:int = None, input_placeholder: str = None, input_value: str = None
    ):
        self.check_if_id_exists(input_id)
        element = {
            "id": input_id,
            "type": "Input.Text",
            "isMultiline": input_is_multiline
        }

        if input_max_length is not None: element["maxLength"] = input_max_length 
        if input_placeholder is not None: element["placeholder"] = input_placeholder 
        if input_value is not None: element["value"] = input_value 
        self.__add_element(element)
    
    """
    add_submit_action_btn() adds the button that submits the 'form' that has been sent as a message. Works like the <input type="submit"> element in html. 
    """
    def add_submit_action_btn(
        self, title: str = "submit"
    ):
        self.check_if_submit_button_exists()
        action = {
            "type": "Action.Submit",
            "title": title
        }
        self.__add_action(action)

    
    """
    The various attribute values can be found at: https://adaptivecards.io/schemas/adaptive-card.json#/definitions/Input.ChoiceSet
    """
    def add_input_choiceset(
        self, input_id: str, input_choices:list=[], input_is_multiselect: bool = False, input_value:str = None
    ):
        self.check_if_id_exists(input_id)
        print("*" * 70)
        print(input_choices)
        print("*" * 70)
        element = {
            "id": input_id,
            "type": "Input.ChoiceSet",
            "choices": input_choices,
            "isMultiSelect": input_is_multiselect
        }

        if input_value is not None: element["value"] = input_value
        self.__add_element(element)
